@@ -1,4 +1,4 @@
-; ACL2 Version 6.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 6.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2013, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -358,14 +358,16 @@ implementations.")
            (apply v rst)))))
 
 (defun system-call (string arguments)
-  #+akcl
+
+; Warning: Keep this in sync with system-call+.
+
+  #+gcl
   (si::system
    (let ((result string))
      (dolist
       (x arguments)
       (setq result (concatenate 'string result " " x)))
      result))
-  #+lucid (user::run-program string :arguments arguments)
   #+lispworks
   (system::call-system
    (let ((result string))
@@ -425,9 +427,9 @@ implementations.")
                   (integerp (cadr status))))
         1 ; just some non-zero exit code here
       (cadr status)))
-  #-(or akcl lucid lispworks allegro cmu sbcl clisp ccl)
+  #-(or gcl lispworks allegro cmu sbcl clisp ccl)
   (declare (ignore string arguments))
-  #-(or akcl lucid lispworks allegro cmu sbcl clisp ccl)
+  #-(or gcl lispworks allegro cmu sbcl clisp ccl)
   (error "SYSTEM-CALL is not yet defined in this Lisp."))
 
 (defun copy-acl2 (dir)
@@ -502,7 +504,9 @@ implementations.")
          (setq source-directory
                (subseq source-directory 0 (1- (length source-directory))))))
   (cond ((not (equal (our-truename (format nil "~a/" source-directory) :safe)
-                     (our-truename "" t)))
+                     (our-truename
+                      ""
+                      "Note: Calling OUR-TRUENAME from COPY-DISTRIBUTION.")))
          (error "We expected to be in the directory~%~s~%~
                  but instead are apparently in the directory~%~s .~%~
                  Either issue, in Unix, the command~%~
@@ -680,26 +684,41 @@ implementations.")
 (defvar *saved-build-date-lst*)
 (defvar *saved-mode*)
 
+(defun svn-revision-from-line (s)
+
+; S is a string such as "$Revision: 1053 $" (as in acl2-startup-info.txt) or
+; "Revision: 1998" (as printed by "svn info").  In general, it is a string for
+; which we want the object that is read immediately after the first #\: .  If
+; there is none, then we return nil.
+
+  (let ((p (position #\: s)))
+    (and p
+         (read-from-string s nil nil :start (1+ p)))))
+
 (defconstant *acl2-svn-revision-string*
-  "~% WARNING: Do not redistribute.  This is NOT an ACL2 release; it is,~
-   ~% rather, an svn distribution, ~a.~
-   ~% The authors of ACL2 consider svn distributions to be experimental.~%")
-
-(defun acl2-svn-revision-string ()
-
-; Put symbol :release into file acl2-startup-info.txt before doing a release.
-; Otherwise, we expect that file to start with a string of the form
-; "$Revision: n $".
-
   (let ((file "acl2-startup-info.txt"))
     (cond ((probe-file file)
            (let ((val (with-open-file
                        (str file :direction :input)
                        (read str))))
              (cond ((eq val :release)
-                    "")
+                    nil)
                    ((stringp val)
-                    (format nil *acl2-svn-revision-string* val))
+                    (let ((n (svn-revision-from-line val)))
+                      (or n (error "Unexpected error in getting svn revision ~
+                                    from string:~%~s~%"
+                                   val))
+                      (format nil
+                              "
+ ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+ + WARNING: This is NOT an ACL2 release; it is svn revision ~a.     +
+ + The authors of ACL2 consider svn distributions to be experimental; +
+ + they may be incomplete, fragile, and unable to pass our own        +
+ + regression.  Bug reports should include the following line:        +
+ +   ACL2 svn revision ~a; community books svn revision ~a        +
+ ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+"
+                              n n "~a")))
                    (t (error "Illegal value in file ~s: ~s"
                              file val)))))
           (t (error "File ~s appears not to exist." file)))))
@@ -713,10 +732,13 @@ implementations.")
     ~% are welcome to redistribute it under certain conditions.  For details,~
     ~% see the LICENSE file distributed with ACL2.~%"
 
-; The following is empty if acl2-startup-info.txt begins with the symbol
-; :release.
+   (or *acl2-svn-revision-string*
 
-   (acl2-svn-revision-string)
+; If acl2-startup-info.txt begins with the symbol :release, then
+; *acl2-svn-revision-string* is a string with ~a expecting to be bound to the
+; svn revision of the books.  Otherwise we put "~a" here to be bound to "".
+
+       "~a")
 
    "~a"
    #+hons
@@ -760,6 +782,22 @@ implementations.")
         #+cltl2 'with-standard-io-syntax
         args))
 
+(defun user-args-string (inert-args &optional (separator '"--"))
+
+; This function is used when saving executable scripts, which may specify that
+; certain command line arguments are not to be processed by Lisp (other than to
+; affect the value of a variable or function such as
+; ccl::*unprocessed-command-line-arguments*; see books/oslib/argv.lisp).  Also
+; see :doc save-exec.  A common convention is that arguments after `--' are not
+; processed by Lisp.
+
+  (cond ((null inert-args)
+         "\"$@\"")
+        ((eq inert-args t)
+         (concatenate 'string separator " \"$@\""))
+        (t
+         (concatenate 'string separator " " inert-args " \"$@\""))))
+
 (defmacro write-exec-file (stream prefix string &rest args)
 
 ; Prefix is generally nil, but can be (string . fmt-args).  String is the
@@ -788,10 +826,92 @@ implementations.")
             ,string
             ,@args)))
 
+(defun proclaim-files (&optional outfilename infilename infile-optional-p)
+
+; IMPORTANT: This function assumes that the defconst and defmacro forms in the
+; given files have already been evaluated.  One way to achieve this state of
+; affairs, of course, is to load the files first.
+
+  (when (and outfilename infilename)
+    (error "It is illegal to supply non-nil values for both optional ~
+            arguments of proclaim-files."))
+  (when (not *do-proclaims*)
+    (return-from proclaim-files nil))
+  (cond
+   (outfilename
+    (format t
+            "Writing proclaim forms for ACL2 source files to file ~s.~%"
+            outfilename))
+   (t
+    (when infilename
+        (cond ((probe-file infilename)
+               (format t
+                       "Loading nontrivial generated file of proclaim forms, ~
+                        ~s...~%"
+                       infilename)
+               (load infilename)
+               (format t
+                       "Completed load of ~s.~%"
+                       infilename)
+               (return-from proclaim-files nil))
+              (infile-optional-p) ; fall through as though infilename is nil
+              (t (error "File ~s is to be loaded by proclaim-files, but does ~
+                         not exist."
+                        infilename))))
+       (format t
+            "Generating and evaluating proclaim forms for ACL2 source ~
+             files.~%")))
+  (let (str)
+    (when outfilename
+      (if (probe-file outfilename)
+          (delete-file outfilename))
+      (or (setq str (safe-open outfilename :direction :output))
+          (error "Unable to open file ~s for output." outfilename)))
+
+; It is tempting to print an in-package form, but we leave that task to
+; proclaim-file, which presumably finds the first form to be an in-package
+; form.
+
+    (dolist (fl *acl2-files*)
+      (proclaim-file (format nil "~a.lisp" fl) str))
+    (when str ; equivalently, when outfilename is non-nil
+      (close str))))
+
+(defun insert-string (s)
+  (cond ((null s) "")
+        (t (concatenate 'string " " s))))
+
+#+gcl
+(defvar *saved-system-banner*
+
+; This variable is only used in GCL 2.6.9 and later, and the following comments
+; pertain only to that case.
+
+; Set this variable to nil before save-exec in order to save an image without a
+; GCL startup banner, as this will leave si::*system-banner* unbound; see
+; below.
+
+; ACL2 keeps this value at nil except when acl2-default-restart unbinds
+; si::*system-banner*, in which case *saved-system-banner* is set to the value
+; of si::*system-banner* just before that unbinding takes place.  When
+; save-exec saves an image, it first checks whether si::*system-banner* is
+; unbound and *saved-system-banner* is non-nil, in which case it sets
+; si::*system-banner* to *saved-system-banner*.  Even if si::*system-banner* is
+; bound, *saved-system-banner* is set to nil before saving an image.
+
+  nil)
+
 #+akcl
 (defun save-acl2-in-akcl-aux (sysout-name gcl-exec-name
                                           write-worklispext
-                                          set-optimize-maximum-pages)
+                                          set-optimize-maximum-pages
+                                          host-lisp-args
+                                          inert-args)
+  (when (and (gcl-version-> 2 6 9 t)
+             *saved-system-banner*)
+    (when (not (boundp 'si::*system-banner*)) ; true, unless user intervened
+      (setq si::*system-banner* *saved-system-banner*))
+    (setq *saved-system-banner* nil))
   (if (and write-worklispext (probe-file "worklispext"))
       (delete-file "worklispext"))
   (let* ((ext "gcl")
@@ -812,7 +932,10 @@ implementations.")
     (if (probe-file gcl-exec-file)
         (delete-file gcl-exec-file))
     (with-open-file (str sysout-name :direction :output)
-                    (write-exec-file str nil "~s $*~%" gcl-exec-file))
+                    (write-exec-file str nil "~s~s ~a~%"
+                                     gcl-exec-file
+                                     (insert-string host-lisp-args)
+                                     (user-args-string inert-args)))
     (cond ((and set-optimize-maximum-pages
                 (boundp 'si::*optimize-maximum-pages*))
 
@@ -961,14 +1084,6 @@ implementations.")
             (lp)))
   (load "akcl-acl2-trace.lisp")
 
-; The following is important so that ACL2 functions are efficient in certain
-; situations.  For example, (aref1 'foo foo n) should avoid boxing a fixnum n.
-
-  (cond (*suppress-compile-build-time*
-         (load "acl2-proclaims.lisp")
-         (load-acl2))
-        (t (proclaim-files)))
-
 ; Return to normal allocation growth.  Keep this in sync with load-acl2, which
 ; had presumably already set the allocation growth to be particularly slow.
 
@@ -979,7 +1094,10 @@ implementations.")
 ; In akcl, at least some versions of it, we cannot call allocate-growth on the
 ; following two types.
 
-          #+gcl contiguous
+; Camm Maguire has told us on 9/22/2013 that certain allocations for contiguous
+; pages, as we now do in acl2.lisp for GCL 2.6.10 and later (which includes GCL
+; 2.6.10pre as of 9/22/2013).
+;          #+gcl contiguous
           #+gcl relocatable
           )
    do
@@ -991,7 +1109,10 @@ implementations.")
     (t (si::allocate-growth type 0 0 0))))
 
 ;  (print "Start (si::gbc nil)") ;debugging GC
-  (si::set-hole-size 500) ; wfs suggestion
+
+; Camm Maguire suggests leaving the hole size alone for GCL 2.6.10pre as of
+; 9/22/2013:
+;  (si::set-hole-size 500) ; wfs suggestion
 
 ; Camm Maguire says (7/04) that "the gc algorithm skips over any pages which
 ; have not been written to since sgc-on was invoked.  So gc really needs to be
@@ -1032,7 +1153,10 @@ implementations.")
                        n))
           do (setq new-hole-size (+ new-hole-size (- n space))))
 ;    (print "Set hole size") ;debugging
-    (si::set-hole-size new-hole-size))
+; Camm Maguire suggests leaving the hole size alone for GCL 2.6.10pre as of
+; 9/22/2013:
+;    (si::set-hole-size new-hole-size)
+    )
 
 ; The calculation above is legacy.  Now we increment the hole size to 20% of
 ; max-pages instead of the default 10%.  Camm Maguire says that "Larger values
@@ -1040,21 +1164,24 @@ implementations.")
 ; part of the virtual (not resident) memory size, rather than being saved to
 ; disk.
 
-  (let ((new-size (floor si:*lisp-maxpages* 5)))
-    (if (< (si:get-hole-size) new-size)
-        (si::set-hole-size new-size)))
+; Camm Maguire suggests leaving the hole size alone for GCL 2.6.10pre as of
+; 9/22/2013:
+;  (let ((new-size (floor si:*lisp-maxpages* 5)))
+;    (if (< (si:get-hole-size) new-size)
+;        (si::set-hole-size new-size)))
 
 ;  (print (true-listp (w *the-live-state*))) ;swap in the world's pages
 
 ;  (print "Save the system") ;debugging
   (when (not do-not-save-gcl)
-    (save-acl2-in-akcl-aux sysout-name gcl-exec-name t t)))
+    (save-acl2-in-akcl-aux sysout-name gcl-exec-name t t nil nil)))
 
 #+akcl
-(defun save-exec-raw (sysout-name)
+(defun save-exec-raw (sysout-name host-lisp-args inert-args)
   (setq *acl2-allocation-alist* nil) ; Don't meddle with allocations.
   (setq *acl2-default-restart-complete* nil)
-  (save-acl2-in-akcl-aux sysout-name sysout-name nil nil))
+  (save-acl2-in-akcl-aux sysout-name sysout-name nil nil host-lisp-args
+                         inert-args))
 
 (defvar *acl2-default-restart-complete* nil)
 
@@ -1082,10 +1209,28 @@ implementations.")
       (setq *default-pathname-defaults* p)))
   nil)
 
+(defvar *print-startup-banner*
+
+; One might want to set this variable to nil in raw Lisp before calling
+; save-exec, in order to avoid seeing startup information.  We do not comment
+; here on whether that is legally appropriate; for example, it suppresses
+; copyright information for ACL2 and, for CCL at least, information about the
+; host Lisp.  We also do not guarantee that this behavior (suppressing printing
+; of startup information) is supported for every host Lisp.
+
+; Note that LD always prints some startup information, regardless of the value
+; of *print-startup-banner*.  To suppress that information, evaluate
+; (set-ld-verbose nil state) in the ACL2 loop.
+
+  t)
+
+(defvar *lp-ever-entered-p* nil)
+
 (defun acl2-default-restart ()
   (if *acl2-default-restart-complete*
       (return-from acl2-default-restart nil))
 
+  (setq *lp-ever-entered-p* nil)
   (#+cltl2
    common-lisp-user::acl2-set-character-encoding
    #-cltl2
@@ -1099,19 +1244,44 @@ implementations.")
 ; In CCL, print greeting now, rather than upon first re-entry to ACL2 loop.
 ; Here we follow a suggestion from Gary Byers.
 
-    (format t "~&Welcome to ~A ~A!~%"
-            (lisp-implementation-type)
-            (lisp-implementation-version))
+    (when *print-startup-banner*
+      (format t "~&Welcome to ~A ~A!~%"
+              (lisp-implementation-type)
+              (lisp-implementation-version)))
     (setq ccl::*inhibit-greeting* t))
+
+  #+gcl
+  (progn
+
+; Some recent versions of GCL (specifically, 2.6.9 in Sept. 2013) do not print
+; the startup banner until we first exit the loop.  So we handle that situation
+; much as we handle a similar issue for CCL above, following GCL source file
+; lsp/gcl_top.lsp.
+
+    (when (and *print-startup-banner*
+               (gcl-version-> 2 6 9 t)
+               (boundp 'si::*system-banner*))
+      (format t si::*system-banner*)
+      (setq *saved-system-banner* si::*system-banner*)
+      (makunbound 'si::*system-banner*)
+      (when (boundp 'si::*tmp-dir*)
+        (format t "Temporary directory for compiler files set to ~a~%"
+                si::*tmp-dir*))))
+
   #+hons (qfuncall acl2h-init)
-  (format t *saved-string*
-          *copy-of-acl2-version*
-          (saved-build-dates :terminal)
-          (cond (*saved-mode*
-                 (format nil "~% Initialized with ~a." *saved-mode*))
-                (t ""))
-          (eval '(latest-release-note-string)) ; avoid possible warning
-          )
+  (when *print-startup-banner*
+    (format t
+            *saved-string*
+            *copy-of-acl2-version*
+            (saved-build-dates :terminal)
+            (if (null *acl2-svn-revision-string*)
+                ""
+              (qfuncall acl2-books-revision))
+            (cond (*saved-mode*
+                   (format nil "~% Initialized with ~a." *saved-mode*))
+                  (t ""))
+            (eval '(latest-release-note-string)) ; avoid possible warning
+            ))
   (maybe-load-acl2-init)
   (eval `(in-package ,*startup-package-name*))
 
@@ -1131,7 +1301,8 @@ implementations.")
 
 #+cmu
 (defun cmulisp-restart ()
-  (extensions::print-herald t)
+  (when *print-startup-banner*
+    (extensions::print-herald t))
   (acl2-default-restart)
   (lp))
 
@@ -1148,7 +1319,8 @@ implementations.")
                   :full-gc t))
 
 #+lispworks
-(defun lispworks-save-exec-aux (sysout-name eventual-sysout-name)
+(defun lispworks-save-exec-aux (sysout-name eventual-sysout-name
+                                            host-lisp-args inert-args)
 
 ; LispWorks support (Dave Fox) pointed out, in the days of LispWorks 4, that we
 ; need to be sure to call (mp:initialize-multiprocessing) when starting up.  Up
@@ -1215,8 +1387,10 @@ implementations.")
 ; changing the underlying Lisp implementation before building ACL2 (again,
 ; presumably based on knowledge of the host Lisp implementation).
 
-                                     "~s -init - -siteinit - $*~%"
-                                     eventual-lw-exec-file))
+                                     "~s -init - -siteinit -~a ~a~%"
+                                     eventual-lw-exec-file
+                                     (insert-string host-lisp-args)
+                                     (user-args-string inert-args)))
     (chmod-executable sysout-name)
     (cond ((and system::*init-file-loaded*
                 system::*complain-about-init-file-loaded*)
@@ -1248,18 +1422,21 @@ implementations.")
       (delete-file "worklispext"))
   (with-open-file (str "worklispext" :direction :output)
                   (format str "lw"))
-  (lispworks-save-exec-aux sysout-name eventual-sysout-name))
+  (lispworks-save-exec-aux sysout-name eventual-sysout-name
+                           nil nil))
 
 #+lispworks
-(defun save-exec-raw (sysout-name)
+(defun save-exec-raw (sysout-name host-lisp-args inert-args)
 
 ; See the comment above about :multiprocessing t.
 
   (setq *acl2-default-restart-complete* nil)
-  (lispworks-save-exec-aux sysout-name sysout-name))
+  (lispworks-save-exec-aux sysout-name sysout-name
+                           host-lisp-args inert-args))
 
 #+cmu
-(defun save-acl2-in-cmulisp-aux (sysout-name core-name)
+(defun save-acl2-in-cmulisp-aux (sysout-name core-name
+                                             host-lisp-args inert-args)
   (let ((eventual-sysout-core
          (unix-full-pathname core-name "core"))
         (sysout-core
@@ -1289,9 +1466,27 @@ implementations.")
                          (t prog1))))
        (write-exec-file str
                         nil
-                        "~s -core ~s -eval '(acl2::cmulisp-restart)' $*~%"
+                        "~s -core ~s -dynamic-space-size ~s -eval ~
+                         '(acl2::cmulisp-restart)'~a ~a~%"
                         prog2
-                        eventual-sysout-core)))
+                        eventual-sysout-core
+
+; In our testing for ACL2 Version_6.2 we found that certification failed for
+; ACL2(h) built on CMUCL for the book tau/bounders/elementary-bounders.lisp,
+; with the error: "CMUCL has run out of dynamic heap space (512 MB)."  This
+; failure doesn't seem to be fully reproduceable, but it seems safest to
+; increase the stack size.  Our CMUCL image, even though on 64-bit linux,
+; reported the following when we tried a value of 2000 here:
+
+; -dynamic-space-size must be no greater than 1632 MBytes.
+
+; Indeed, we have exceeded that in a version of community book
+; books/centaur/gl/solutions.lisp using ACL2(h) built on CMUCL.  So we use the
+; maximum possible value just below.
+
+                        1632
+                        (insert-string host-lisp-args)
+                        (user-args-string inert-args))))
     (chmod-executable sysout-name)
     (system::gc)
     (extensions::save-lisp sysout-core :load-init-file nil :site-init nil
@@ -1308,12 +1503,12 @@ implementations.")
       (delete-file "worklispext"))
   (with-open-file (str "worklispext" :direction :output)
                   (format str "core"))
-  (save-acl2-in-cmulisp-aux sysout-name core-name))
+  (save-acl2-in-cmulisp-aux sysout-name core-name nil nil))
 
 #+cmu
-(defun save-exec-raw (sysout-name)
+(defun save-exec-raw (sysout-name host-lisp-args inert-args)
   (setq *acl2-default-restart-complete* nil)
-  (save-acl2-in-cmulisp-aux sysout-name sysout-name))
+  (save-acl2-in-cmulisp-aux sysout-name sysout-name host-lisp-args inert-args))
 
 #+sbcl
 (defvar *sbcl-dynamic-space-size*
@@ -1363,7 +1558,17 @@ implementations.")
   #-(and x86-64 hons) 2000)
 
 #+sbcl
-(defun save-acl2-in-sbcl-aux (sysout-name core-name)
+(defvar *sbcl-contrib-dir* nil)
+
+#+sbcl
+(defun save-acl2-in-sbcl-aux (sysout-name core-name
+                                          host-lisp-args
+                                          toplevel-args
+                                          inert-args)
+
+; Note that host-lisp-args specifies what the SBCL manual calls "runtime
+; options", while toplevel-args is what it calls "toplevel options".
+
   (declaim (optimize (sb-ext:inhibit-warnings 3)))
   (let ((eventual-sysout-core
          (unix-full-pathname core-name "core"))
@@ -1386,19 +1591,23 @@ implementations.")
 
         ("~a~%"
          (let ((contrib-dir
-                (and (boundp 'sb-ext::*core-pathname*)
-                     (ignore-errors
-                      (let* ((core-dir
-                              (pathname-directory sb-ext::*core-pathname*))
-                             (contrib-dir-pathname
-                              (and (equal (car (last core-dir))
-                                          "output")
-                                   (make-pathname
-                                    :directory
-                                    (append (butlast core-dir 1)
-                                            (list "contrib"))))))
-                        (and (probe-file contrib-dir-pathname)
-                             (namestring contrib-dir-pathname)))))))
+                (or
+                 *sbcl-contrib-dir*
+                 (and (boundp 'sb-ext::*core-pathname*)
+                      (ignore-errors
+                        (let* ((core-dir
+                                (pathname-directory
+                                 sb-ext::*core-pathname*))
+                               (contrib-dir-pathname
+                                (and (equal (car (last core-dir))
+                                            "output")
+                                     (make-pathname
+                                      :directory
+                                      (append (butlast core-dir 1)
+                                              (list "contrib"))))))
+                          (and (probe-file contrib-dir-pathname)
+                               (setq *sbcl-contrib-dir*
+                                     (namestring contrib-dir-pathname)))))))))
            (if contrib-dir
                (format nil
                        "export SBCL_HOME=~s"
@@ -1411,26 +1620,28 @@ implementations.")
 ; to be stack overflow from fmt0, which is not tail recursive.  More recently,
 ; community book centaur/misc/defapply.lisp causes a stack overflow even with
 ; --control-stack-size 4 (though that might disappear after we added (comp t)
-; in a couple of places).  So we use 8 instead of 4.
+; in a couple of places).  Yet more recently, community books
+; books/centaur/regression/common.lisp and books/centaur/tutorial/intro.lisp
+; fail with --control-stack-size 8, due to calls of def-gl-clause-processor.
+; So we use --control-stack-size 16.  We might increase 16 to 32 or greater in
+; the future.
 
 ; See *sbcl-dynamic-space-size* for an explanation of the --dynamic-space-size
 ; setting below.
 
-        "~s --dynamic-space-size ~s --control-stack-size 8 --core ~s~a ~
-         --eval '(acl2::sbcl-restart)'"
+; Note that --no-userinit was introduced into SBCL in Version 0.9.13, hence has
+; been part of SBCL since 2007 (perhaps earlier).  So when Jared Davis pointed
+; out this option to us after ACL2 Version_6.2, we started using it in place of
+; " --userinit /dev/null", which had not worked on Windows.
+
+        "~s --dynamic-space-size ~s --control-stack-size 16 --core ~s~a ~
+         --end-runtime-options --no-userinit --eval '(acl2::sbcl-restart)'~a ~a~%"
         prog
         *sbcl-dynamic-space-size*
         eventual-sysout-core
-
-; We have found (August 2008) that SBCL 1.0.13 on Windows does not like
-; "--userinit /dev/null".  But we have been using this option for quite some
-; time on other platforms.  So we conditionalize here, but we are open to
-; suggestions from SBCL users on a better way to handle this.
-
-        #-mswindows
-        " --userinit /dev/null"
-        #+mswindows
-        "")))
+        (insert-string host-lisp-args)
+        (insert-string toplevel-args)
+        (user-args-string inert-args "--end-toplevel-options"))))
     (chmod-executable sysout-name)
     ;; In SBCL 0.9.3 the read-only space is too small for dumping ACL2 on x86,
     ;; so we have to specify :PURIFY NIL. This will unfortunately result in
@@ -1447,24 +1658,20 @@ implementations.")
    (setq *saved-mode* mode)
    (if (probe-file "worklispext")
        (delete-file "worklispext"))
-   (cond ((and *suppress-compile-build-time*
-               (probe-file "acl2-proclaims.lisp"))
-; We do the following load when *suppress-compile-build-time* in save-acl2, but
-; it's harmless enough to do it again here.
-          (load "acl2-proclaims.lisp"))
-         (t (proclaim-files)))
    (with-open-file (str "worklispext" :direction :output)
                    (format str "core"))
-   (save-acl2-in-sbcl-aux sysout-name core-name)))
+   (save-acl2-in-sbcl-aux sysout-name core-name nil nil nil)))
 
 #+sbcl
-(defun save-exec-raw (sysout-name)
+(defun save-exec-raw (sysout-name host-lisp-args toplevel-args inert-args)
   (with-warnings-suppressed
    (setq *acl2-default-restart-complete* nil)
-   (save-acl2-in-sbcl-aux sysout-name sysout-name)))
+   (save-acl2-in-sbcl-aux sysout-name sysout-name host-lisp-args toplevel-args
+                          inert-args)))
 
 #+allegro
-(defun save-acl2-in-allegro-aux (sysout-name dxl-name)
+(defun save-acl2-in-allegro-aux (sysout-name dxl-name
+                                             host-lisp-args inert-args)
   (excl:gc t) ; Suggestions are welcome for better gc call(s)!
   #+(and allegro-version>= (version>= 4 3))
   (progn
@@ -1502,11 +1709,13 @@ implementations.")
 ; Allegro 6.2 to avoid getting Allegro copyright information printed upon :q if
 ; we start up in the ACL2 read-eval-print loop.
 
-;         "~s -I ~s -L ~s~%"
+;         "~s -I ~s -L ~s ~s~%"
 
-          "~s -I ~s $*~%"
+          "~s -I ~s~s ~a~%"
           (system::command-line-argument 0)
-          eventual-sysout-dxl))
+          eventual-sysout-dxl
+          (insert-string host-lisp-args)
+          (user-args-string inert-args)))
         (chmod-executable sysout-name)
         (excl:dumplisp :name sysout-dxl)))
     #-(and allegro-version>= (version>= 5 0))
@@ -1528,12 +1737,12 @@ implementations.")
   (with-open-file (str "worklispext" :direction :output)
                   (format str "dxl"))
   (load "allegro-acl2-trace.lisp") ; Robert Krug's trace patch
-  (save-acl2-in-allegro-aux sysout-name dxl-name))
+  (save-acl2-in-allegro-aux sysout-name dxl-name nil nil))
 
 #+allegro
-(defun save-exec-raw (sysout-name)
+(defun save-exec-raw (sysout-name host-lisp-args inert-args)
   (setq *acl2-default-restart-complete* nil)
-  (save-acl2-in-allegro-aux sysout-name sysout-name))
+  (save-acl2-in-allegro-aux sysout-name sysout-name host-lisp-args inert-args))
 
 (defun rc-filename (dir)
   (concatenate 'string dir ".acl2rc"))
@@ -1557,7 +1766,7 @@ implementations.")
                   (ACL2::LP)~%")))))
 
 #+clisp
-(defun save-acl2-in-clisp-aux (sysout-name mem-name)
+(defun save-acl2-in-clisp-aux (sysout-name mem-name host-lisp-args inert-args)
   (let ((save-dir (our-pwd))
         (eventual-sysout-mem
          (unix-full-pathname mem-name "mem"))
@@ -1572,7 +1781,7 @@ implementations.")
      (str sysout-name :direction :output)
      (write-exec-file str
                       nil
-                      "~s -i ~s -p ACL2 -M ~s -m ~dMB -E ISO-8859-1 $*~%"
+                      "~s -i ~s -p ACL2 -M ~s -m ~dMB -E ISO-8859-1~a ~a~%"
                       (or (ext:getenv "LISP") "clisp")
                       (rc-filename save-dir)
                       eventual-sysout-mem
@@ -1588,7 +1797,9 @@ implementations.")
 ; write-exec-file seems to put a decimal point after the number when using ~s,
 ; and CLISP complains about that when starting up.
 
-                      10))
+                      10
+                      (insert-string host-lisp-args)
+                      (user-args-string inert-args)))
     (chmod-executable sysout-name)
     (ext:gc)
     (ext:saveinitmem sysout-mem
@@ -1607,15 +1818,18 @@ implementations.")
       (delete-file "worklispext"))
   (with-open-file (str "worklispext" :direction :output)
                   (format str "mem"))
-  (save-acl2-in-clisp-aux sysout-name mem-name))
+  (save-acl2-in-clisp-aux sysout-name mem-name nil nil))
 
 #+clisp
-(defun save-exec-raw (sysout-name)
+(defun save-exec-raw (sysout-name host-lisp-args inert-args)
   (setq *acl2-default-restart-complete* nil)
-  (save-acl2-in-clisp-aux sysout-name sysout-name))
+  (save-acl2-in-clisp-aux sysout-name sysout-name host-lisp-args inert-args))
 
 #+ccl
-(defun save-acl2-in-ccl-aux (sysout-name core-name)
+(defun save-acl2-in-ccl-aux (sysout-name core-name
+                                         &optional
+                                         (host-lisp-args nil save-exec-p)
+                                         inert-args)
   (let* ((ccl-program0
           (or (car ccl::*command-line-argument-list*) ; Gary Byers suggestion
               (error "Unable to determine CCL program pathname!")))
@@ -1667,18 +1881,50 @@ implementations.")
                                      default-dir)
                            "")))
 
+; See the section on "reading characters from files" in file acl2.lisp for an
+; explanation of the -K argument below.
+
 ; It is probably important to use -e just below instead of :toplevel-function,
 ; at least for #+hons.  Jared Davis and Sol Swords have told us that it seems
 ; that with :toplevel-function one gets a new "toplevel" thread at start-up,
 ; which "plays badly with the thread-local hash tables that make up the hons
 ; space".
 
-; See the section on "reading characters from files" in file acl2.lisp for an
-; explanation of the -K argument below.
-
-                      "~s -I ~s -K ISO-8859-1 -e \"(acl2::acl2-default-restart)\" $*~%"
+                      "~s -I ~s~a -K ISO-8859-1 -e ~
+                       \"(acl2::acl2-default-restart)\"~a ~a~%"
                       ccl-program
-                      core-name))
+                      core-name
+                      (if save-exec-p
+
+; For an ACL2 built from sources, the saved script will include "-Z 64M"; see
+; comment below.  But with save-exec, no -Z option will be written.  The new
+; script can then be expected to invoke ACL2 with the same stack sizes as did
+; the original (which had -Z 64M explicitly), unless an explicit -Z option is
+; given to save-exec or globals such as
+; ccl::*initial-listener-default-control-stack-size* (see community book
+; books/centaur/ccl-config.lsp) are set before the save-exec call.
+
+; Turning now to the case of building from sources, as opposed to save-exec:
+
+; We use -Z 64M even though the default for -Z (as of mid-2013) is 2M, in order
+; to get larger stacks.  We have ample evidence that a larger stack would be
+; useful: an ACL2 example from David Russinoff in August 2013 for which 8M was
+; insufficient (defining a constant function with body '(...), a quoted list of
+; length 65536; our own x86 model requiring 4M for an ACL2 proof using
+; def-gl-thm; and more generally, Centaur's routine use of large stacks,
+; equivalent to -Z 256M.  Not surprisingly, we that performance was not hurt
+; using a larger stack size, for two pairs of ACL2(h) regressions as follows.
+; We ran one pair of runs on a Linux system with 32GB of RAM, and one pair of
+; runs on a MacBook Pro with 8GB of RAM, all in August 2013.  For each pair we
+; ran with -Z 64M and also omitting -Z (equivalent to using -Z 2M).  Our main
+; concern was potentially larger backtraces when using (set-debugger-enable
+; :bt), as during a regression.  We solved that by restricting backtrace counts
+; using *ccl-print-call-history-count*.
+
+                          ""
+                        " -Z 64M")
+                      (insert-string host-lisp-args)
+                      (user-args-string inert-args)))
     (chmod-executable sysout-name)
     (ccl::gc)
     (ccl:save-application core-name)))
@@ -1686,19 +1932,13 @@ implementations.")
 #+ccl
 (defun save-acl2-in-ccl (sysout-name &optional mode core-name)
   (setq *saved-mode* mode)
-  (cond ((and *suppress-compile-build-time*
-              (probe-file "acl2-proclaims.lisp"))
-; We do the following load when *suppress-compile-build-time* in save-acl2, but
-; it's harmless enough to do it again here.
-         (load "acl2-proclaims.lisp"))
-        (t (proclaim-files)))
   (load "openmcl-acl2-trace.lisp")
   (save-acl2-in-ccl-aux sysout-name core-name))
 
 #+ccl
-(defun save-exec-raw (sysout-name)
+(defun save-exec-raw (sysout-name host-lisp-args inert-args)
   (setq *acl2-default-restart-complete* nil)
-  (save-acl2-in-ccl-aux sysout-name sysout-name))
+  (save-acl2-in-ccl-aux sysout-name sysout-name host-lisp-args inert-args))
 
 ; Since saved-build-date-string is avoided for MCL, we avoid the following too
 ; (which is not applicable to MCL sessions anyhow).
@@ -1722,8 +1962,6 @@ implementations.")
 ; (ccl::save-application "acl2-image" :size (expt 2 24))
 ; for the Mac.
 
-  (when (and *suppress-compile-build-time* (probe-file "acl2-proclaims.lisp"))
-    (load "acl2-proclaims.lisp"))
   (load-acl2)
   (setq *saved-build-date-lst*
 
@@ -1765,33 +2003,12 @@ implementations.")
   (error "We do not know how to save ACL2 in this Common Lisp.")
   (format t "Saving of ACL2 is complete.~%"))
 
-(defun proclaim-files (&optional outfilename)
-
-; IMPORTANT:  This function assumes that the defconst forms in the
-; given files have already been evaluated.  One way to achieve this
-; state of affairs, of course, is to load the files first.
-
-  (if outfilename
-      (format t
-              "Writing proclaim forms for ACL2 source files to file ~s.~%"
-              outfilename)
-    (format t
-              "Generating and evaluating proclaim forms for ACL2 source ~
-               files.~%"))
-  (let (str)
-    (when outfilename
-      (if (probe-file outfilename)
-          (delete-file outfilename))
-      (or (setq str (safe-open outfilename :direction :output))
-          (error "Unable to open file ~s for output." outfilename))
-      (format str "(in-package \"ACL2\")~%"))
-    (dolist (fl *acl2-files*)
-      (proclaim-file (format nil "~a.lisp" fl) str))))
-
 (defun generate-acl2-proclaims ()
+
+; See the section "PROCLAIMING" in acl2-fns.lisp.
+
   (let ((filename "acl2-proclaims.lisp"))
-    (cond ((and *suppress-compile-build-time*
-                *do-proclaims*)
+    (cond (*do-proclaims*
            (format t "Beginning load-acl2 and initialize-acl2 on behalf of ~
                       generate-acl2-proclaims.~%")
            (load-acl2 t)
@@ -1808,20 +2025,6 @@ implementations.")
                     "; No proclaims are generated here for this host Lisp.~%"))
            nil))))
 
-(defun acl2 nil
-  (let ((*readtable* *acl2-readtable*)
-        (extension (if *suppress-compile-build-time*
-                       *lisp-extension*
-                     *compiled-file-extension*)))
-    (dolist (name (remove "defpkgs" *acl2-files* :test #'equal))
-            (if (equal name "proof-checker-pkg")
-                (load "proof-checker-pkg.lisp")
-              (load-compiled (make-pathname :name name
-                                            :type extension))))
-    (load "defpkgs.lisp")
-    (in-package "ACL2")
-    "ACL2"))
-
 ; The following avoids core being dumped in certain circumstances
 ; resulting from very hard errors.
 
