@@ -1,4 +1,4 @@
-; ACL2 Version 6.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 6.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2013, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -90,9 +90,9 @@
         ACL2-ORACLE ACL2-OUTPUT-CHANNEL-PACKAGE
         ACL2-PACKAGE ACONS ACTIVE-RUNEP
         ADD-BINOP ADD-CUSTOM-KEYWORD-HINT
-        ADD-DEFAULT-HINTS
-        ADD-DEFAULT-HINTS! ADD-INCLUDE-BOOK-DIR
-        ADD-INVISIBLE-FNS ADD-MACRO-ALIAS ADD-MACRO-FN
+        ADD-DEFAULT-HINTS ADD-DEFAULT-HINTS! ADD-INCLUDE-BOOK-DIR
+        ADD-INVISIBLE-FNS ADD-LD-KEYWORD-ALIAS ADD-LD-KEYWORD-ALIAS!
+        ADD-MACRO-ALIAS ADD-MACRO-FN
         ADD-MATCH-FREE-OVERRIDE ADD-NTH-ALIAS
         ADD-OVERRIDE-HINTS ADD-OVERRIDE-HINTS!
         ADD-PAIR ADD-PAIR-PRESERVES-ALL-BOUNDP
@@ -296,7 +296,7 @@
         KNOWN-PACKAGE-ALIST KNOWN-PACKAGE-ALISTP
         KNOWN-PACKAGE-ALISTP-FORWARD-TO-TRUE-LIST-LISTP-AND-ALISTP
         KWOTE
-        KWOTE-LST LAMBDA LAST LD LD-ERROR-ACTION
+        KWOTE-LST LAMBDA LAST LAST-PROVER-STEPS LD LD-ERROR-ACTION
         LD-ERROR-TRIPLES LD-EVISC-TUPLE
         LD-KEYWORD-ALIASES LD-MISSING-INPUT-OK LD-POST-EVAL-PRINT
         LD-PRE-EVAL-FILTER LD-PRE-EVAL-PRINT
@@ -330,7 +330,7 @@
         META-EXTRACT-GLOBAL-FACT META-EXTRACT-GLOBAL-FACT+ META-EXTRACT-RW+-TERM
         MFC MIN MINIMAL-THEORY MINUSP
         MOD MOD-EXPT MONITOR MONITORED-RUNES
-        MORE MORE! MORE-DOC MUST-BE-EQUAL
+        MORE MORE! MORE-DOC MSG MUST-BE-EQUAL
         MUTUAL-RECURSION MUTUAL-RECURSION-GUARDP
         MV MV? MV-LET MV?-LET MV-LIST
         MV-NTH NAT-LISTP NATP NEEDS-SLASHES NEVER-MEMOIZE NEWLINE
@@ -462,11 +462,11 @@
         SET-GAG-MODE SET-GUARD-CHECKING
         SET-IGNORE-DOC-STRING-ERROR
         SET-IGNORE-OK SET-INHIBIT-OUTPUT-LST
-        SET-INHIBIT-WARNINGS
+        SET-INHIBIT-WARNINGS SET-INHIBIT-WARNINGS!
         SET-INHIBITED-SUMMARY-TYPES
         SET-INVISIBLE-FNS-TABLE
         SET-IPRINT SET-IRRELEVANT-FORMALS-OK
-        SET-LD-KEYWORD-ALIASES
+        SET-LD-KEYWORD-ALIASES SET-LD-KEYWORD-ALIASES!
         SET-LD-REDEFINITION-ACTION
         SET-LD-SKIP-PROOFS
         SET-LD-SKIP-PROOFSP SET-LET*-ABSTRACTION
@@ -549,7 +549,7 @@
         SYMBOL-NAME-INTERN-IN-PACKAGE-OF-SYMBOL
         SYMBOL-PACKAGE-NAME SYMBOLP
         SYMBOLP-INTERN-IN-PACKAGE-OF-SYMBOL
-        SYNP SYNTAXP SYS-CALL SYS-CALL-STATUS
+        SYNP SYNTAXP SYS-CALL SYS-CALL+ SYS-CALL-STATUS
         T T-STACK T-STACK-LENGTH T-STACK-LENGTH1
         TABLE TABLE-ALIST TAKE
         TAU-DATA TAU-DATABASE
@@ -594,7 +594,7 @@
         UPPER-CASE-P-FORWARD-TO-ALPHA-CHAR-P
         USER-STOBJ-ALIST USER-STOBJ-ALIST1
         VALUE-TRIPLE VERBOSE-PSTACK VERIFY
-        VERIFY-GUARDS VERIFY-GUARDS-FORMULA
+        VERIFY-GUARDS VERIFY-GUARDS+ VERIFY-GUARDS-FORMULA
         VERIFY-TERMINATION W WALKABOUT
         WARNING! WATERFALL-PARALLELISM WATERFALL-PRINTING
         WET WITH-FAST-ALIST WITH-GUARD-CHECKING
