@@ -1,4 +1,4 @@
-; ACL2 Version 6.2 -- A Computational Logic for Applicative Common Lisp
+; ACL2 Version 6.3 -- A Computational Logic for Applicative Common Lisp
 ; Copyright (C) 2013, Regents of the University of Texas
 
 ; This version of ACL2 is a descendent of ACL2 Version 1.9, Copyright
@@ -520,6 +520,9 @@ sometimes be lost due to user interrupts and aborts.~/
 
 Performance Notes
 
+~l[hons-acons] for how the final ~ilc[cdr] of a fast alist can be used as a
+size hint or as a name for reports printed by calling ~ilc[fast-alist-summary].
+
 The keys of fast alists are always ~il[normed].  Why?  In Common Lisp,
 equal-based hashing is relatively slow, so to allow the use of eql-based hash
 tables, ~ilc[hons-acons] and ~ilc[hons-get] always ~ilc[hons-copy] the keys
@@ -788,8 +791,9 @@ from an alist or to copy ~il[fast-alists].~/
 
 This ~il[documentation] topic relates to the experimental extension of ACL2
 supporting hash cons, fast alists, and memoization; ~pl[hons-and-memoization].
+It assumes familiarity with fast alists; ~pl[fast-alists].
 
-Logically, ~c[hons-shrink-alist] is defined as follows:
+Logically, ~c[(hons-shrink-alist alist ans)] is defined as follows:
 
 ~bv[]
  (cond ((atom alist)
@@ -812,11 +816,36 @@ reversed.  This can be useful as a way to clean up any unnecessary bindings in
 alist, or as a way to obtain a \"deep copy\" of a fast alist that can extended
 independently from the original while maintaining discipline.
 
+Note that hons-shrink-alist is potentially expensive, for the following two
+reasons.
+~bq[]
+
+o The alist is copied, dropping any shadowed pairs.  This process will require
+a hash table lookup for each entry in the alist; and it will require creating a
+new alist, which uses additional memory.
+
+o Unless ans is a fast alist that is stolen (see below), a new hash table is
+created, which uses additional memory.  This hash table is populated in time
+that is linear in the number of unique keys in the alist.
+~eq[]
+
 When ans is not an atom, good discipline requires that it is a fast alist.  In
 this case, ~c[hons-shrink-alist] steals the hash table for ans and extends it
 with all of the bindings in alist that are not in ans.  From the perspective of
 ~ilc[hons-assoc-equal], you can think of the resulting alist as being basically
-similar to ~c[(append ans alist)], but in a different order.~/~/"
+similar to ~c[(append ans alist)], but in a different order.
+
+Note that when ans is not a fast alist (e.g., ans is an atom) then such
+stealing does not take place.
+
+A common idiom is to replace an alist by the result of shrinking it, in which
+case it is best to free the input alist, for example as follows.
+~bv[]
+  (let ((alist (fast-alist-free-on-exit alist
+                                        (hons-shrink-alist alist nil))))
+    ...)
+~ev[]
+~l[fast-alist-free-on-exit] and ~pl[fast-alist-free].~/~/"
 
   ;; Has an under-the-hood implementation
   (cond ((atom alist)
@@ -884,7 +913,9 @@ it odd to ever prove a theorem about it.
 
 Under the hood, ~c[fast-alist-free] removes the hash table associated with this
 alist, if one exists.  This effectively converts the alist into an ordinary
-alist.~/
+alist.
+
+Also ~pl[fast-alist-free-on-exit].~/
 
 Because there is no automatic mechanism for freeing the hash tables used in
 fast alists, to avoid memory leaks you should manually free any alists that
@@ -1011,16 +1042,16 @@ tail-recursive call returns.~/~/")
   ":Doc-Section Hons-and-Memoization
 Free a fast alist after the completion of some form.~/
 
-Logically, ~c[(fast-alist-free-on-exit name form)] is the identity and returns
-~c[form].
+Logically, ~c[(fast-alist-free-on-exit alist form)] is the identity and returns
+~c[form].  Also ~pl[fast-alist-free].
 
 Under the hood, this essentially expands to:
 ~bv[]
  (prog1 form
-        (fast-alist-free name))
+        (fast-alist-free alist))
 ~ev[]
 
-In other words, ~c[name] is not freed immediately, but instead remains a fast
+In other words, ~c[alist] is not freed immediately, but instead remains a fast
 alist until the form completes.  This may be useful when you are writing code
 that uses a fast alist but has many return points.
 
@@ -1166,5 +1197,13 @@ Deprecated.  Alias for ~ilc[fast-alist-free].~/~/"
             (:executable-counterpart flush-hons-get-hash-table-link)
             ))
 
+(defun remove-keyword (word l)
+  (declare (xargs :guard (and (keywordp word)
+                              (keyword-value-listp l))))
+  (cond ((endp l) nil)
+        ((eq word (car l))
+         (remove-keyword word (cddr l)))
+        (t (list* (car l) (cadr l) (remove-keyword word (cddr l))))))
+
 ; For some additional helper functions and lemmas, see the community books
 ; books/misc/hons-help.lisp and books/misc/hons-help2.lisp.
